
/* Name: main.c
 * Project: V-USB MIDI device on Low-Speed USB
 * Author: Martin Homuth-Rosemann
 * Creation Date: 2008-03-11
 * Copyright: (c) 2008 by Martin Homuth-Rosemann.
 * License: GPL.
 *
 */

#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include "mouse.h"
#include "usbdrv.h"
#include "oddebug.h"
#include "usb.h"

#define F_CPU 16000000UL

static void hardwareInit(void)
{
	uchar i, j;

	/* activate pull-ups except on USB lines */
	USB_CFG_IOPORT =
	    (uchar) ~ ((1 << USB_CFG_DMINUS_BIT) |
		       (1 << USB_CFG_DPLUS_BIT));
	/* all pins input except USB (-> USB reset) */


	USBDDR = (1 << USB_CFG_DMINUS_BIT) | (1 << USB_CFG_DPLUS_BIT);


	j = 0;
	while (--j) {		/* USB Reset by device only required on Watchdog Reset */
		i = 0;
		while (--i);	/* delay >10ms for USB reset */
	}

	USBDDR = 0;		/*  remove USB reset condition */



}



int main(void)
{


	wdt_enable(WDTO_1S);
	hardwareInit();
	odDebugInit();
	usbInit();
	init_mouse();


	sei();
	
	// only ADC channel 6 and channel 7 are used

	while(1) {		/* main event loop */
		wdt_reset();
		usbPoll();



	}
	return 0;
}
